@@ -25,7 +25,6 @@
 #include <gtk/gtk.h>
 
 #include <evince-document.h>
-#include "ev-page-action.h"
 #include "ev-page-action-widget.h"
 
 /* Widget we pass back */
@@ -246,35 +245,48 @@ ev_page_action_widget_init (EvPageActionWidget *action_widget)
 	gtk_container_add (GTK_CONTAINER (action_widget), hbox);
 	gtk_widget_show (hbox);
 
+        gtk_widget_set_sensitive (GTK_WIDGET (action_widget), FALSE);
 	gtk_widget_show (GTK_WIDGET (action_widget));
 }
 
+static void
+ev_page_action_widget_set_document (EvPageActionWidget *action_widget,
+                                    EvDocument         *document)
+{
+        if (document) {
+                g_object_ref (document);
+                gtk_widget_set_sensitive (GTK_WIDGET (action_widget), ev_document_get_n_pages (document) > 0);
+        }
+
+        if (action_widget->signal_id > 0) {
+                g_signal_handler_disconnect (action_widget->doc_model,
+                                             action_widget->signal_id);
+                action_widget->signal_id = 0;
+        }
+
+        if (action_widget->document)
+                g_object_unref (action_widget->document);
+        action_widget->document = document;
+        if (!action_widget->document)
+                return;
+
+        action_widget->signal_id =
+                g_signal_connect (action_widget->doc_model,
+                                  "page-changed",
+                                  G_CALLBACK (page_changed_cb),
+                                  action_widget);
+
+        ev_page_action_widget_set_current_page (action_widget,
+                                                ev_document_model_get_page (action_widget->doc_model));
+        ev_page_action_widget_update_max_width (action_widget);
+}
+
 static void
 ev_page_action_widget_document_changed_cb (EvDocumentModel    *model,
 					   GParamSpec         *pspec,
 					   EvPageActionWidget *action_widget)
 {
-	EvDocument *document = ev_document_model_get_document (model);
-
-	g_object_ref (document);
-	if (action_widget->document)
-		g_object_unref (action_widget->document);
-	action_widget->document = document;
-
-	if (action_widget->signal_id > 0) {
-		g_signal_handler_disconnect (action_widget->doc_model,
-					     action_widget->signal_id);
-		action_widget->signal_id = 0;
-	}
-	action_widget->signal_id =
-		g_signal_connect (action_widget->doc_model,
-                                  "page-changed",
-                                  G_CALLBACK (page_changed_cb),
-                                  action_widget);
-
-	ev_page_action_widget_set_current_page (action_widget,
-						ev_document_model_get_page (model));
-        ev_page_action_widget_update_max_width (action_widget);
+        ev_page_action_widget_set_document (action_widget, ev_document_model_get_document (model));
 }
 
 void
@@ -289,6 +301,7 @@ ev_page_action_widget_set_model (EvPageActionWidget *action_widget,
 	g_object_add_weak_pointer (G_OBJECT (model),
 				   (gpointer)&action_widget->doc_model);
 
+        ev_page_action_widget_set_document (action_widget, ev_document_model_get_document (model));
 	g_signal_connect (model, "notify::document",
 			  G_CALLBACK (ev_page_action_widget_document_changed_cb),
 			  action_widget);
@@ -310,20 +323,36 @@ ev_page_action_widget_finalize (GObject *object)
 		action_widget->doc_model = NULL;
 	}
 
-	if (action_widget->document) {
-		g_object_unref (action_widget->document);
-		action_widget->document = NULL;
-	}
+        ev_page_action_widget_set_document (action_widget, NULL);
 
 	G_OBJECT_CLASS (ev_page_action_widget_parent_class)->finalize (object);
 }
 
 static void
-ev_page_action_widget_class_init (EvPageActionWidgetClass *class)
+ev_page_action_widget_get_preferred_width (GtkWidget *widget,
+                                           gint      *minimum_width,
+                                           gint      *natural_width)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS (class);
+        GtkWidget *child;
+
+        *minimum_width = *natural_width = 0;
+
+        child = gtk_bin_get_child (GTK_BIN (widget));
+        if (!child || !gtk_widget_get_visible (child))
+                return;
+
+        gtk_widget_get_preferred_width (child, minimum_width, natural_width);
+        *natural_width = *minimum_width;
+}
+
+static void
+ev_page_action_widget_class_init (EvPageActionWidgetClass *klass)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+        GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
 	object_class->finalize = ev_page_action_widget_finalize;
+        widget_class->get_preferred_width = ev_page_action_widget_get_preferred_width;
 
 	widget_signals[WIDGET_ACTIVATE_LINK] =
 		g_signal_new ("activate_link",
@@ -511,7 +540,7 @@ ev_page_action_widget_update_links_model (EvPageActionWidget *proxy, GtkTreeMode
 	GtkEntryCompletion *completion;
 	GtkCellRenderer *renderer;
 
-	if (!model)
+	if (!model || model == proxy->model)
 		return;
 
 	/* Magik */
