@@ -206,6 +206,7 @@ struct _EvView {
 	/* Focus */
 	EvMapping *focused_element;
 	guint focused_element_page;
+	guint child_focus_idle_id;
 
 	/* Synctex */
 	EvMapping *synctex_result;
@@ -286,5 +287,11 @@ void _ev_view_set_selection   (EvView   *view,
 			       GdkPoint *start_point,
 			       GdkPoint *end_point);
 
+void _ev_view_set_focused_element (EvView *view,
+				   EvMapping *element_mapping,
+				   gint page);
+void _ev_view_focus_form_field    (EvView      *view,
+				   EvFormField *field);
+
 #endif  /* __EV_VIEW_PRIVATE_H__ */
 
