@@ -38,19 +38,20 @@ typedef struct _EvToolbarClass   EvToolbarClass;
 typedef struct _EvToolbarPrivate EvToolbarPrivate;
 
 struct _EvToolbar {
-        GtkToolbar base_instance;
+        GtkHeaderBar base_instance;
 
         EvToolbarPrivate *priv;
 };
 
 struct _EvToolbarClass {
-        GtkToolbarClass base_class;
+        GtkHeaderBarClass base_class;
 };
 
 GType      ev_toolbar_get_type           (void);
 GtkWidget *ev_toolbar_new                (EvWindow *window);
 gboolean   ev_toolbar_has_visible_popups (EvToolbar *ev_toolbar);
 void       ev_toolbar_action_menu_popup  (EvToolbar *ev_toolbar);
+GtkWidget *ev_toolbar_get_page_selector  (EvToolbar *ev_toolbar);
 
 G_END_DECLS
 
